type PlatformName = 'mac' | 'win' | 'linux' | 'key';
declare const getCurrentPlatform: (navigator?: {
    platform: string;
}) => PlatformName;

type PlatformKeyboardShortcut = Partial<Record<PlatformName, string>>;

declare const FALLTHROUGH = "fallthrough";
type KeyCommandFunction = () => void | typeof FALLTHROUGH;
type KeyCommandOptions = {
    allowInInput?: boolean;
};
type KeyCommandObject = KeyCommandOptions & {
    command: KeyCommandFunction;
};
type KeyCommand = KeyCommandObject | KeyCommandFunction;
type KeyMap = Record<string, KeyCommand>;
type KeyShortcuts = [string | PlatformKeyboardShortcut, KeyCommand][] | KeyMap;
declare function createKeyMap(shortcuts: KeyShortcuts, platformName: PlatformName): KeyMap;

declare const handleKeyboardEvent: (event: KeyboardEvent, platformName: PlatformName, shortcuts: KeyShortcuts, options?: {
    invokedWithinInput?: boolean;
}) => void;
declare function elementShouldHandleOwnShortcut(target: EventTarget | null): boolean;

declare const KEYBOARD_SHORTCUT_PASSTHROUGH_CLASS = "ignore-global-events";
type KeyEventName = "keydown" | "keyup" | "keypress";
interface KeyboardShortcutOptions {
    eventName?: KeyEventName;
}
declare function useKeyboardShortcuts(shortcuts: KeyShortcuts, options?: KeyboardShortcutOptions): void;

declare function parseKeyName(name: string, platform: PlatformName): {
    key: string;
    modifiers: KeyModifiers;
};
declare function normalizeKeyName(name: string, platform: PlatformName): string;
declare function getShortcutDisplayParts(name: string, platform: PlatformName): {
    separator?: string;
    keys: string[];
};
interface KeyModifiers {
    altKey: boolean;
    ctrlKey: boolean;
    metaKey: boolean;
    shiftKey: boolean;
}
declare function prependModifiers(name: string, event: KeyModifiers, useShift: boolean): string;

export { FALLTHROUGH, KEYBOARD_SHORTCUT_PASSTHROUGH_CLASS, type KeyCommand, type KeyCommandFunction, type KeyCommandObject, type KeyCommandOptions, type KeyMap, type KeyModifiers, type KeyShortcuts, type PlatformName, createKeyMap, elementShouldHandleOwnShortcut, getCurrentPlatform, getShortcutDisplayParts, handleKeyboardEvent, normalizeKeyName, parseKeyName, prependModifiers, useKeyboardShortcuts };
