"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/index.ts
var src_exports = {};
__export(src_exports, {
  FALLTHROUGH: () => FALLTHROUGH,
  KEYBOARD_SHORTCUT_PASSTHROUGH_CLASS: () => KEYBOARD_SHORTCUT_PASSTHROUGH_CLASS,
  createKeyMap: () => createKeyMap,
  elementShouldHandleOwnShortcut: () => elementShouldHandleOwnShortcut,
  getCurrentPlatform: () => getCurrentPlatform,
  getShortcutDisplayParts: () => getShortcutDisplayParts,
  handleKeyboardEvent: () => handleKeyboardEvent,
  normalizeKeyName: () => normalizeKeyName,
  parseKeyName: () => parseKeyName,
  prependModifiers: () => prependModifiers,
  useKeyboardShortcuts: () => useKeyboardShortcuts
});
module.exports = __toCommonJS(src_exports);

// src/hooks.ts
var import_react = require("react");

// src/platform.ts
var getCurrentPlatform = (navigator2) => typeof navigator2 === "undefined" ? "key" : /Mac/.test(navigator2.platform) ? "mac" : /Win/.test(navigator2.platform) ? "win" : /Linux|X11/.test(navigator2.platform) ? "linux" : "key";

// src/hooks.ts
var KEYBOARD_SHORTCUT_PASSTHROUGH_CLASS = "ignore-global-events";
var listenerElement = typeof window !== "undefined" ? window : null;
var platformName = typeof navigator !== "undefined" ? getCurrentPlatform(navigator) : "key";
function useKeyboardShortcuts(shortcuts, options = {}) {
  const eventName = options.eventName ?? "keydown";
  const shortcutsRef = (0, import_react.useRef)(shortcuts);
  (0, import_react.useEffect)(() => {
    shortcutsRef.current = shortcuts;
  });
  (0, import_react.useEffect)(() => {
    const handler = (event) => {
      const invokedWithinInput = elementShouldHandleOwnShortcut(event.target);
      handleKeyboardEvent(event, platformName, shortcutsRef.current, {
        invokedWithinInput
      });
    };
    listenerElement?.addEventListener(eventName, handler);
    return () => {
      listenerElement?.removeEventListener(eventName, handler);
    };
  }, [eventName]);
}

// src/shortcuts.ts
var import_w3c_keyname = require("w3c-keyname");

// src/names.ts
function parseKeyName(name, platform) {
  const parts = name.split(/-(?!$)/);
  let result = parts[parts.length - 1];
  if (result === "Space")
    result = " ";
  const modifiers = {
    altKey: false,
    ctrlKey: false,
    shiftKey: false,
    metaKey: false
  };
  if (parts.length === 1) {
    if (/^(cmd|meta)$/i.test(result))
      return { key: "Meta", modifiers };
    else if (/^alt$/i.test(result))
      return { key: "Alt", modifiers };
    else if (/^(ctrl|control)$/i.test(result))
      return { key: "Ctrl", modifiers };
    else if (/^shift$/i.test(result))
      return { key: "Shift", modifiers };
    else if (/^mod$/i.test(result)) {
      if (platform === "mac")
        return { key: "Meta", modifiers };
      else
        return { key: "Ctrl", modifiers };
    }
  }
  for (let i = 0; i < parts.length - 1; ++i) {
    const mod = parts[i];
    if (/^(cmd|meta)$/i.test(mod))
      modifiers.metaKey = true;
    else if (/^alt$/i.test(mod))
      modifiers.altKey = true;
    else if (/^(ctrl|control)$/i.test(mod))
      modifiers.ctrlKey = true;
    else if (/^shift$/i.test(mod))
      modifiers.shiftKey = true;
    else if (/^mod$/i.test(mod)) {
      if (platform === "mac")
        modifiers.metaKey = true;
      else
        modifiers.ctrlKey = true;
    } else
      throw new Error("Unrecognized modifier name: " + mod);
  }
  return {
    key: result,
    modifiers
  };
}
function normalizeKeyName(name, platform) {
  const parsed = parseKeyName(name, platform);
  let result = parsed.key;
  if (parsed.modifiers.altKey)
    result = "Alt-" + result;
  if (parsed.modifiers.ctrlKey)
    result = "Ctrl-" + result;
  if (parsed.modifiers.metaKey)
    result = "Meta-" + result;
  if (parsed.modifiers.shiftKey)
    result = "Shift-" + result;
  return result;
}
var modifierDisplayName = {
  altKey: "Alt",
  ctrlKey: "Ctrl",
  metaKey: "\u229E",
  shiftKey: "Shift"
};
var macModiferDisplayName = {
  altKey: "\u2325",
  ctrlKey: "^",
  metaKey: "\u2318",
  shiftKey: "\u21E7"
};
function getModifierDisplayName(name, platform) {
  return platform === "mac" ? macModiferDisplayName[name] : modifierDisplayName[name];
}
function getShortcutDisplayParts(name, platform) {
  const { key, modifiers } = parseKeyName(name, platform);
  return {
    separator: platform === "mac" ? void 0 : "-",
    keys: [
      modifiers.altKey && getModifierDisplayName("altKey", platform),
      modifiers.ctrlKey && getModifierDisplayName("ctrlKey", platform),
      modifiers.shiftKey && getModifierDisplayName("shiftKey", platform),
      modifiers.metaKey && getModifierDisplayName("metaKey", platform),
      key.toUpperCase()
    ].filter((x) => !!x)
  };
}
function prependModifiers(name, event, useShift) {
  if (event.altKey)
    name = "Alt-" + name;
  if (event.ctrlKey)
    name = "Ctrl-" + name;
  if (event.metaKey)
    name = "Meta-" + name;
  if (useShift !== false && event.shiftKey)
    name = "Shift-" + name;
  return name;
}

// src/shortcuts.ts
function getPlatformShortcutName(shortcut, platformName2) {
  const platformKey = shortcut[platformName2] || shortcut.key;
  return platformKey ? normalizeKeyName(platformKey, platformName2) : void 0;
}
var modifierKeyNames = /* @__PURE__ */ new Set(["Alt", "Control", "Meta", "Shift"]);
function getEventShortcutNames(event, platformName2) {
  const eventKeyName = (0, import_w3c_keyname.keyName)(event);
  const isChar = eventKeyName.length === 1 && eventKeyName !== " ";
  const eventModifiers = {
    altKey: event.altKey,
    shiftKey: event.shiftKey,
    ctrlKey: event.ctrlKey,
    metaKey: event.metaKey
  };
  const eventShortcutName = normalizeKeyName(
    modifierKeyNames.has(eventKeyName) ? eventKeyName : prependModifiers(eventKeyName, eventModifiers, !isChar),
    platformName2
  );
  let shortcutNames = [eventShortcutName];
  const baseKeyName = import_w3c_keyname.base[event.keyCode];
  if (isChar) {
    let alternateShortcutName;
    if (
      // Holding these keys may change the key that gets entered,
      // e.g. pressing "Alt" + "a" will enter "å". However, we can look
      // at the basename to get the actual key pressed.
      (event.shiftKey || event.altKey || event.metaKey) && baseKeyName && baseKeyName !== eventShortcutName
    ) {
      alternateShortcutName = normalizeKeyName(
        prependModifiers(baseKeyName, eventModifiers, true),
        platformName2
      );
    } else if (event.shiftKey) {
      alternateShortcutName = normalizeKeyName(
        prependModifiers(eventShortcutName, eventModifiers, true),
        platformName2
      );
    }
    if (alternateShortcutName && !shortcutNames.includes(alternateShortcutName)) {
      shortcutNames.push(alternateShortcutName);
    }
  }
  return shortcutNames;
}

// src/keyMap.ts
var FALLTHROUGH = "fallthrough";
function createKeyMap(shortcuts, platformName2) {
  const shortcutsArray = Array.isArray(shortcuts) ? shortcuts : Object.entries(shortcuts);
  return Object.fromEntries(
    shortcutsArray.flatMap(([key, command]) => {
      const platformShortcutName = getPlatformShortcutName(
        typeof key === "string" ? { key } : key,
        platformName2
      );
      return platformShortcutName ? [[platformShortcutName, command]] : [];
    })
  );
}

// src/events.ts
var matchShortcut = (event, platformName2, shortcuts, options = {}) => {
  const keyMap = createKeyMap(shortcuts, platformName2);
  const eventShortcutNames = getEventShortcutNames(event, platformName2);
  const matchingName = eventShortcutNames.find((name) => name in keyMap);
  if (!matchingName)
    return;
  const command = keyMap[matchingName];
  const commandObject = typeof command === "function" ? { command } : command;
  if (options.invokedWithinInput && !commandObject.allowInInput)
    return;
  return commandObject.command;
};
var handleKeyboardEvent = (event, platformName2, shortcuts, options = {}) => {
  const command = matchShortcut(event, platformName2, shortcuts, options);
  if (!command)
    return;
  const result = command();
  if (result !== FALLTHROUGH) {
    event.preventDefault();
    event.stopImmediatePropagation();
  }
};
function elementShouldHandleOwnShortcut(target) {
  if (!(target instanceof HTMLElement))
    return true;
  let element = target;
  const closestContentEditable = element.closest("[contenteditable]");
  if (closestContentEditable) {
    element = closestContentEditable;
  }
  if (element.classList.contains(KEYBOARD_SHORTCUT_PASSTHROUGH_CLASS)) {
    return false;
  }
  if (element instanceof HTMLInputElement || element instanceof HTMLTextAreaElement || element instanceof HTMLIFrameElement || element.hasAttribute("contenteditable")) {
    return true;
  }
  return false;
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  FALLTHROUGH,
  KEYBOARD_SHORTCUT_PASSTHROUGH_CLASS,
  createKeyMap,
  elementShouldHandleOwnShortcut,
  getCurrentPlatform,
  getShortcutDisplayParts,
  handleKeyboardEvent,
  normalizeKeyName,
  parseKeyName,
  prependModifiers,
  useKeyboardShortcuts
});
//# sourceMappingURL=index.js.map